unit uExtendedDbGrid;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, db, DBGrids, Grids, Graphics, Themes
  , Types, TypInfo, controls, stdctrls, lclintf, LCLType
  ,uExtendedDBGrid_constant, uExtendedDBGrid_type, unit_campocombodbgrid;

type
    TTextTitleChange = procedure(Sender: TObject; aCol: integer; aFieldName: string; aValue: string) of object;

type

  { TDBGrid }

  TDBGrid = Class(DBGrids.TDBGrid)
  private
    FOnTextTitleChange: TTextTitleChange;
    FSmartFilter: boolean;
    FTextFilter: boolean;
    FStartedTitleRowHeights: integer;
    FLastMouseDown_Is_Title_Button: boolean;
    TitleFilterTextValues: array of TTextFilterOfTitle;
    LastDrawColumnPosition: array of TLastColumnPosition;
    Edit1 : TEdit;
    ComboBoxF: TComboBox;
    ComboBox1: TComboBox;
    lista_campi: TArrayTParametriAddComboBox;
    indici: array of TReverseIndex;
    ListOfComboColumn: array of TComboColumn;
    function GetIdxFromTitleFilterTextValues(index_column: integer):integer;
    function GetIdxFromLastDrawColumnPosition(index_column: integer):integer;
    procedure OnTextFilterKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure OnTextFilterEditingDone(Sender: TObject);
    procedure OnTextFilterExit(Sender: TObject);
    procedure OnTextFilterEnter(Sender: TObject);
    procedure OnComboBox1Enter(Sender: TObject);
    procedure OnComboBox1KeyPress(Sender: TObject; var Key: char);
    procedure OnComboBox1EditingDone(Sender: TObject);
    procedure HideOtherComponent();
    function ColumnIsCombo(aFieldName: string; var Style: TComboBoxStyle): boolean;
    procedure OnChangeComboFilter(Sender: TObject);
  protected
    procedure DrawAllRows; virtual;
    procedure DrawExtendedSmallButtonCell(const aCol, aRow: Integer; aRect: TRect; const aState: TGridDrawState);
    procedure MouseDown(Button: TMouseButton; Shift:TShiftState; X,Y:Integer); override;
    procedure MyDrawColumnTitle(Sender: TObject; const Rect: TRect; DataCol: Integer;Column: TColumn; State: TGridDrawState);
    procedure MySelectEditor(Sender: TObject; Column: TColumn; var MyEditor: TWinControl);
  published
    property LastMouseDown_Is_Title_Button: boolean read FLastMouseDown_Is_Title_Button Write FLastMouseDown_Is_Title_Button;
    property SmartFilter: boolean read FSmartFilter Write FSmartFilter;
    Property TextFilter: boolean read FTextFilter Write FTextFilter;
    property OnTextFilterChange: TTextTitleChange read FOnTextTitleChange write FOnTextTitleChange;
    procedure DrawCell(ACol, ARow: Longint; ARect: TRect; AState: TGridDrawState); override;
  public
    destructor Destroy;override;
    constructor Create(AOwner: TComponent);override;
    procedure SetTitleCaption(aCol: integer; MyCaption: string);
    procedure SetCheckColumn(aCol: integer; CheckedValue: string; UncheckedValue: string);
    procedure SetAlternateColor(aColor: TColor);
    procedure SetColumnCombo(aCol: integer; aFieldName: string; Style: TComboBoxStyle;
      indexs: TExtendedDBGridIntegerArray; values: TExtendedDBGridStringArray);
    procedure InitColumnDBCombo();
    procedure SetColumnDBCombo(newfieldname: string; fieldname: string;
      DataSetList: TDataSet; keylist: string; fieldlist: string);
    procedure UpdateColumnDBCombo();
    procedure ClearColumnsCombo;
    function GetFilterText(aCol: integer): string;
  End;

implementation

{ TDBGrid }

function TDBGrid.GetIdxFromTitleFilterTextValues(index_column: integer
  ): integer;
var
   i: integer;
   esci: boolean;
begin
     i:=0;
     esci:=false;
     while (i<Length(Self.TitleFilterTextValues)) and (esci=false) do
     begin
          if Self.TitleFilterTextValues[i].index_column = index_column then
          begin
               esci:=true;
          end else begin
               Inc(i);
          end;
     end;

     if esci=false then
        i:=-1;
     result:=i;
end;

function TDBGrid.GetIdxFromLastDrawColumnPosition(index_column: integer
  ): integer;
var
   i: integer;
   esci: boolean;
begin
     i:=0;
     esci:=false;
     while (i<Length(Self.LastDrawColumnPosition)) and (esci=false) do
     begin
          if Self.LastDrawColumnPosition[i].index_column = index_column then
          begin
               esci:=true;
          end else begin
               Inc(i);
          end;
     end;

     if esci=false then
        i:=-1;
     result:=i;
end;

procedure TDBGrid.OnTextFilterKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
     if (Key = VK_TAB) or (Key = VK_RETURN) or (Key = VK_ESCAPE) then
     begin
       Self.OnTextFilterEditingDone(Sender);  //effettuo il salvataggio
       //proseguo con l'uscita
       Self.SetFocus;
       SendMessage(Self.Handle, VK_DOWN, Key, 0);
       TEdit(Sender).Visible:=false;
     end
end;

procedure TDBGrid.OnTextFilterExit(Sender: TObject);
begin
     Self.OnTextFilterEditingDone(Sender);
end;

procedure TDBGrid.OnTextFilterEnter(Sender: TObject);
begin

end;

procedure TDBGrid.OnTextFilterEditingDone(Sender: TObject);
var
   colonna, idx,lCol: integer;
begin
     if dgIndicator in Self.Options then
        lCol:=1
     else
         lCol:=0;
     colonna:=ptrint( TEdit(Sender).Tag );
     idx:=Self.GetIdxFromTitleFilterTextValues(colonna);
     if idx <>-1 then
     begin
           Self.TitleFilterTextValues[idx].filter:=TEdit(Sender).Text;
           Self.TitleFilterTextValues[idx].label_filter:=TEdit(Sender).Text;
           Self.DrawRow(0);
           Self.InvalidateRow(0);
           Self.FOnTextTitleChange(Sender,colonna, Self.DataSource.DataSet.Fields[colonna-lCol].FieldName,TEdit(Sender).Text);
     end;
end;

procedure TDBGrid.OnComboBox1Enter(Sender: TObject);
var
   aCol : integer;
   i,j  : integer;
   esci : boolean;
begin
     aCol:=Self.SelectedColumn.Index;
     i:=0;
     esci:=false;
     while (i<Length(Self.ListOfComboColumn)) and (esci = false) do
     begin

          if Self.ListOfComboColumn[i].aCol=aCol then
          begin
               esci:=true;

               TComboBox(Sender).Clear;
               for j:=0 to Length(Self.ListOfComboColumn[i].values)-1 do
               begin
                    TComboBox(Sender).Items.Add(Self.ListOfComboColumn[i].values[j]);
               end;

          end else begin
               Inc(i);
          end;


     end;

end;

procedure TDBGrid.OnComboBox1KeyPress(Sender: TObject; var Key: char);
var
  nome: string;
  valore: integer;
  nome_funzione: string;
  script: string;
  errori: string;
  indice: integer;
begin
  {with Self.RxDBGrid1 do
  begin
    if nome_campo_rimozione_filtro = ComboBox1_Entrato_In_Campo then
    begin
      nome := stringReplace(nome_campo_rimozione_filtro, SUFFISSO_CAMPI_RUN_TIME, '', [RfReplaceAll]);
      if TComboBox(Sender).ItemIndex <> -1 then
      begin
        valore := PtrInt(TComboBox(Sender).Items.Objects[TComboBox(Sender).ItemIndex]);
      end
      else
      begin
        valore := StrToIntDef(DBLookupComboBox1_Key, 0);
      end;

      indice := TComboBox(Sender).ItemIndex;
      if ((valore = valore_combobox1_onenter) or (indice = -1)) then
      begin
        //Qui non faccio nulla perchè vuol dire che non ho cambiato il
        //valore nella combo
      end
      else
      begin
        if not (DataSource.State in [dsEdit, dsInsert]) then
        begin
          Datasource.Edit;
        end;
        DataSource.DataSet.FieldByName(nome).AsInteger := valore;

        DBLookupComboBox1_Key   := IntToStr(valore);   //DataSource.DataSet.FieldByName(nome_campo_rimozione_filtro).KeyFields;
        DBLookupComboBox1_Value := TComboBox(Sender).Text; //IntToStr(valore);


        nome_funzione := 'TFORM1_DBLookupComboBox1EditingDone';

        //script da eseguire a inizio procedura
        script := Frm_Splash.GetPSScript(nome_funzione, 0);
        Self.EseguiCodicePas(script, errori, False, False);
        if AbortPS = 0 then
        begin
          //qui se voglio ci metto del codice

          //script da eseguire a fine procedura
          script := Frm_Splash.GetPSScript(nome_funzione, 1);
          Self.EseguiCodicePas(script, errori, False, False);
        end;
      end;

    end;
  end;  }
end;

procedure TDBGrid.OnComboBox1EditingDone(Sender: TObject);
begin

end;

destructor TDBGrid.Destroy;
begin
  //Edit1.Free;
  //ComboBox1.Free;
  inherited;
end;

constructor TDBGrid.Create(AOwner: TComponent);
begin
  //init variable
  FStartedTitleRowHeights := -1;
  Self.OnDrawColumnTitle:=@MyDrawColumnTitle;
  Self.OnSelectEditor:=@MySelectEditor;

  inherited Create(AOwner);

  Edit1:=TEdit.Create(AOwner);
  Edit1.Top:=1;
  Edit1.Left:=1;
  Edit1.Visible:=false;

  ComboBoxF:=TComboBox.Create(AOwner);
  ComboBoxF.Top:=1;
  ComboBoxF.Left:=1;
  ComboBoxF.Visible:=false;

  ComboBox1:=TComboBox.Create(AOwner);
  ComboBox1.Top:=1;
  ComboBox1.Left:=1;
  ComboBox1.Visible:=false;
end;

procedure TDBGrid.DrawAllRows;
begin
     with Self.GCache.VisibleGrid do
     begin
          Bottom := Bottom + 1;
     end;
     inherited;
end;

procedure TDBGrid.DrawExtendedSmallButtonCell(const aCol, aRow: Integer;
  aRect: TRect; const aState: TGridDrawState);
var
  details: TThemedElementDetails;
  bRect: TRect;
  mytext: string;
begin
  bRect:=aRect;
  bRect.Left:=bRect.Right-TITLE_BTN_WIDTH;
  InflateRect(bRect, -2, 0);
  if gdPushed in aState then
    Details := ThemeServices.GetElementDetails(tbPushButtonPressed)
  else
  if gdHot in aState then
    Details := ThemeServices.GetElementDetails(tbPushButtonHot)
  else
    Details := ThemeServices.GetElementDetails(tbPushButtonNormal);
  ThemeSErvices.DrawElement(Canvas.Handle, Details, bRect, nil);

  mytext:='...';
  Canvas.TextRect(bRect, bRect.Left + trunc((bRect.Right-bRect.Left-Canvas.TextWidth(mytext))/2) , trunc((bRect.Bottom-bRect.Top)/2), mytext);
end;

procedure TDBGrid.MouseDown(Button: TMouseButton; Shift: TShiftState; X,
  Y: Integer);
var
   aRect, bRect: TRect;
   aCol, aRow: integer;
   idx_filtro: integer;
   lCol      : integer;
   x1,y1,x2,y2, iQuart: integer;
begin

  if dgIndicator in Self.Options then
       lCol:=1
  else
       lCol:=0;

  Self.LastMouseDown_Is_Title_Button:=false;
  Self.MouseToCell(X,Y,aCol,aRow);

  if (ARow = 0) and (Button=mbLeft) then
  begin

       aRect:=Self.CellRect(aCol,aRow);
       if (Self.SmartFilter) then
       begin
            aRect.Left:=aRect.Right-TITLE_BTN_WIDTH;

            if (X>aRect.Left)and(X<aRect.Right) then
               if (Y>aRect.Top)and(Y<aRect.Bottom) then
                  if (Self.Columns[aCol-lCol].ButtonStyle<>cbsCheckboxColumn) then
                     Self.LastMouseDown_Is_Title_Button:=true;
       end;


       if (Self.LastMouseDown_Is_Title_Button=false) and (Self.TextFilter=true) then
       begin
            idx_filtro := GetIdxFromTitleFilterTextValues(aCol);
            if idx_filtro<>-1 then
            begin
                 if (X>Trunc((aRect.Bottom-aRect.Top)/2) ) then
                           if (Y>Trunc((aRect.Right-aRect.Left)/2) ) then
                           begin

                                aRect:=Self.CellRect(aCol,aRow);

                                iQuart := (aRect.Bottom - aRect.Top) div 2;

                                bRect.Left:=aRect.Left + 2;
                                bRect.Top:=aRect.Top + (aRect.Bottom - aRect.Top - iQuart);
                                if Self.SmartFilter then
                                   bRect.Right := aRect.Right-TITLE_BTN_WIDTH-2
                                else
                                    bRect.Right := aRect.Right-2;
                                bRect.Bottom := aRect.Bottom - 2;

                                TitleFilterTextValues[idx_filtro].last_x_filter:=aRect.Left;
                                TitleFilterTextValues[idx_filtro].last_y_filter:=aRect.Top;

                                if (Self.Columns[aCol-lCol].ButtonStyle=cbsCheckboxColumn) then
                                begin
                                     //ho cliccato nel filtro testuale
                                     with ComboBoxF do
                                     begin
                                           Parent := Self;
                                           Tag := aCol;
                                           Caption := TitleFilterTextValues[idx_filtro].label_filter;
                                           Items.Clear;
                                           Items.Add('');
                                           Items.Add(CAPTION_CHECK);
                                           if TitleFilterTextValues[idx_filtro].label_filter=CAPTION_CHECK then
                                              ItemIndex:=Items.Count-1;
                                           Items.Add(CAPTION_UNCHECK);
                                           if TitleFilterTextValues[idx_filtro].label_filter=CAPTION_UNCHECK then
                                              ItemIndex:=Items.Count-1;
                                           Left := bRect.Left;
                                           Top := bRect.Top;
                                           Width := bRect.Right - bRect.Left;
                                           Height := bRect.Bottom - bRect.Top;
                                           Visible := True;
                                           OnChange := @OnChangeComboFilter;
                                           ComboBoxF.SetFocus;
                                     end;
                                end
                                else
                                begin
                                     //ho cliccato nel filtro testuale
                                     with Edit1 do
                                     begin
                                           Parent := Self;
                                           Tag := aCol;
                                           Text := TitleFilterTextValues[idx_filtro].label_filter;
                                           Left := bRect.Left;
                                           Top := bRect.Top;
                                           Width := bRect.Right - bRect.Left;
                                           Height := bRect.Bottom - bRect.Top;
                                           Visible := True;
                                           OnKeyDown := @OnTextFilterKeyDown;
                                           OnExit := @OnTextFilterExit;
                                           OnEnter := @OnTextFilterEnter;
                                           OnEditingDone := @OnTextFilterEditingDone;
                                           Edit1.SetFocus;
                                     end;
                                end;


                           end;


            end;
       end;


       Self.DrawRow(aRow);
       Self.InvalidateRow(aRow);
  end;

  inherited MouseDown(Button, Shift, X, Y);
end;

procedure TDBGrid.MyDrawColumnTitle(Sender: TObject; const Rect: TRect;
  DataCol: Integer; Column: TColumn; State: TGridDrawState);
var
   bRect: TRect;
   x1,y1,x2,y2,iQuart: integer;
   idx_filtro: integer;
   lCol: integer;
begin
     if Self.TextFilter then
     begin

          if dgIndicator in Self.Options then
             lCol:=1
          else
              lCol:=0;

          idx_filtro := GetIdxFromTitleFilterTextValues(DataCol+lCol); //al datacol aggiungo + 1 perchè sembra che il drawcolumntitle parta da 0 mentre le altre da 1
          if idx_filtro = -1 then
          begin
               SetLength(TitleFilterTextValues, Length(TitleFilterTextValues)+1);
               TitleFilterTextValues[Length(TitleFilterTextValues)-1].index_column:=DataCol+lCol; //al datacol aggiungo + 1 perchè sembra che il drawcolumntitle parta da 0 mentre le altre da 1
               idx_filtro:=Length(TitleFilterTextValues)-1;
          end;

          //-----------------------------
          //questo mi sovrascrive il titolo della colonna perchè starebbe sempre al centro
          //verticalmente, ma io voglio che stia nel top
          TDBGrid(Sender).Canvas.FillRect(Rect.Left, Rect.Top, Rect.Right, Rect.Bottom);
          //Ora ridisegno il testo dove dico io
          bRect.Left:=Rect.Left;
          bRect.Top:=Rect.Top;
          bRect.Right:=Rect.Right;
          bRect.Bottom:=Rect.Top + trunc((Rect.Bottom-Rect.Top)/2) - 1;
          TDBGrid(Sender).Canvas.TextRect(bRect, bRect.Left, bRect.Top ,Column.Title.Caption);
          //-----------------------------

          iQuart := (Rect.Bottom - Rect.Top) div 2;

          bRect.Left:=Rect.Left + 2;
          bRect.Top:=Rect.Top + (Rect.Bottom - Rect.Top - iQuart);
          if Self.SmartFilter then
             bRect.Right := Rect.Right-TITLE_BTN_WIDTH-2
          else
              bRect.Right := Rect.Right-2;
          bRect.Bottom := Rect.Bottom - 2;

          x1:=bRect.Left;
          y1:=bRect.Top;
          x2:=bRect.Right;
          y2:=bRect.Bottom;


          TDBGrid(Sender).Canvas.Brush.Style := bsSolid;
          TDBGrid(Sender).Canvas.Brush.Color := clWhite;
          TDBGrid(Sender).Canvas.Pen.Style   := psSolid;
          TDBGrid(Sender).Canvas.Pen.Color   := clBlue;
          TDBGrid(Sender).Canvas.FillRect(x1,y1,x2,y2);
          TDBGrid(Sender).Canvas.TextRect(bRect, bRect.Left, bRect.Top , TitleFilterTextValues[idx_filtro].filter );

     end;
end;

procedure TDBGrid.MySelectEditor(Sender: TObject; Column: TColumn;
  var MyEditor: TWinControl);
var
  //app_colonna: TModuloGriglia;
  filtro: string;
  i: integer;
  Style: TComboBoxStyle;
begin

  if (Self.SelectedField.DataType = ftMemo) then
  begin
    {GridCellMemo1.BoundsRect := TRXDBGrid(Sender).SelectedFieldRect;
    GridCellMemo1.Text := Column.Field.AsString;
    MyEditor := GridCellMemo1;}
  end
  else if (Self.SelectedField.DataType = ftDate) or (Self.SelectedField.DataType = ftDateTime) then
  begin
    {Self.RxDateEdit1.BoundsRect := TRXDBGrid(Sender).SelectedFieldRect;
    Self.RxDateEdit1.Date := TRXDBGrid(Sender).SelectedField.AsDateTime;
    MyEditor := Self.RxDateEdit1;}
  end
  else if (Self.SelectedField.DataType = ftInteger) or (Self.SelectedField.DataType = ftSmallInt) or (Self.SelectedField.DataType = ftFloat) or (Self.SelectedField.DataType = ftLargeInt) then
  begin


       if ColumnIsCombo(Self.SelectedColumn.FieldName, Style) then
       begin
         Self.ComboBox1.BoundsRect := Self.SelectedFieldRect;
         Self.ComboBox1.Text := Column.Field.AsString;
         Self.ComboBox1.OnEnter := @OnComboBox1Enter;
         Self.ComboBox1.OnKeyPress := @OnComboBox1KeyPress;
         Self.ComboBox1.OnEditingDone := @OnComboBox1EditingDone;
         //Self.ComboBox1.Style:=Style;
         Self.ComboBox1.Visible:=true;
         MyEditor := Self.ComboBox1;
       end;


    {app_colonna := Frm_Splash.GetTipoGrigliaByNomeColonna(Self.modulo, Column.Field.FieldName);
    if ((app_colonna.modulo = Self.modulo) and (app_colonna.is_readonly = False) and (app_colonna.is_spinedit)) then
    begin

      SpinEdit_Grid1.MinValue := app_colonna.spinedit_minvalue;
      SpinEdit_Grid1.MaxValue := app_colonna.spinedit_maxvalue;
      SpinEdit_Grid1.DecimalPlaces := app_colonna.spinedit_decimal;
      SpinEdit_Grid1.Increment := app_colonna.spinedit_increment;
      SpinEdit_Grid1.BoundsRect := TRXDBGrid(Sender).SelectedFieldRect;
      SpinEdit_Grid1.Value := Column.Field.AsInteger;

      MyEditor := SpinEdit_Grid1;
    end;}

  end
  else
  begin



  end;
end;

procedure TDBGrid.DrawCell(ACol, ARow: Longint; ARect: TRect;
  AState: TGridDrawState);
var
  lCol: integer;
  idx: integer;
begin

     //-----------------------------------------------------------------
     //usato per intercettare lo scorrimento a destra o a sinistra
     idx:=GetIdxFromLastDrawColumnPosition(aCol);
     if idx=-1 then
     begin
          SetLength(LastDrawColumnPosition, Length(LastDrawColumnPosition)+1);
          LastDrawColumnPosition[Length(LastDrawColumnPosition)-1].index_column:=aCol; //al datacol aggiungo + 1 perchè sembra che il drawcolumntitle parta da 0 mentre le altre da 1
          idx:=Length(LastDrawColumnPosition)-1;
     end;

     if (aRect.Left<>LastDrawColumnPosition[idx].last_left) then
     begin
          //vuol dire che stiamo scorrendo a destra o a sinistra
          Self.HideOtherComponent();
     end;
     LastDrawColumnPosition[idx].last_left:=arect.Left;
     //-----------------------------------------------------------------

     inherited;

     if dgIndicator in Self.Options then
        lCol:=1
     else
         lCol:=0;


     if (ARow = 0) and (gdFixed in AState) and (aCol>=lCol) then
     begin

          if (Self.SmartFilter = true) and (Self.Columns[aCol-lCol].ButtonStyle<>cbsCheckboxColumn) then
          begin
               DrawExtendedSmallButtonCell(aCol, aRow, aRect, aState);
          end;

          if Self.TextFilter = true then
          begin
               if Self.FStartedTitleRowHeights < 0 then
               begin
                    Self.FStartedTitleRowHeights:=Self.RowHeights[aRow];
               end;
               Self.RowHeights[aRow]:=Self.FStartedTitleRowHeights + Self.DefaultRowHeight;
          end;
     end;

end;

procedure TDBGrid.SetTitleCaption(aCol: integer; MyCaption: string);
begin
     Self.Columns[aCol].Title.Caption:=MyCaption;
end;

procedure TDBGrid.SetCheckColumn(aCol: integer; CheckedValue: string;
  UncheckedValue: string);
begin
     Self.Columns[aCol].ButtonStyle:=cbsCheckboxColumn;
     Self.Columns[aCol].ValueChecked:=CheckedValue;
     Self.Columns[aCol].ValueUnchecked:=UncheckedValue;
end;

procedure TDBGrid.SetAlternateColor(aColor: TColor);
begin
     Self.AlternateColor:=aColor;
end;

procedure TDBGrid.SetColumnCombo(aCol: integer; aFieldName: string;
  Style: TComboBoxStyle; indexs: TExtendedDBGridIntegerArray;
  values: TExtendedDBGridStringArray);
var
  i: integer;
begin

     SetLength(ListOfComboColumn,Length(ListOfComboColumn)+1);
     ListOfComboColumn[Length(ListOfComboColumn)-1].aCol:=aCol;
     ListOfComboColumn[Length(ListOfComboColumn)-1].fieldName:=aFieldName;
     ListOfComboColumn[Length(ListOfComboColumn)-1].Style:=Style;

     SetLength(ListOfComboColumn[Length(ListOfComboColumn)-1].indexs,0);
     for i:=0 to Length(indexs)-1 do
     begin
          SetLength(ListOfComboColumn[Length(ListOfComboColumn)-1].indexs, Length(ListOfComboColumn[Length(ListOfComboColumn)-1].indexs)+1);
          ListOfComboColumn[Length(ListOfComboColumn)-1].indexs[Length(ListOfComboColumn[Length(ListOfComboColumn)-1].indexs)-1]:=indexs[i];
     end;
     SetLength(ListOfComboColumn[Length(ListOfComboColumn)-1].values,0);
     for i:=0 to Length(values)-1 do
     begin
          SetLength(ListOfComboColumn[Length(ListOfComboColumn)-1].values, Length(ListOfComboColumn[Length(ListOfComboColumn)-1].values)+1);
          ListOfComboColumn[Length(ListOfComboColumn)-1].values[Length(ListOfComboColumn[Length(ListOfComboColumn)-1].values)-1]:=values[i];
     end;

end;

procedure TDBGrid.InitColumnDBCombo();
begin
     SetLength(lista_campi,0);
     SetLength(indici,0);
end;

procedure TDBGrid.SetColumnDBCombo(newfieldname: string; fieldname: string;
  DataSetList: TDataSet; keylist: string; fieldlist: string);
begin

     SetLength(lista_campi, Length(lista_campi)+1);
     lista_campi[Length(lista_campi)-1].NomeNuovoCampo:=newfieldname;
     lista_campi[Length(lista_campi)-1].MyDataSet:=Self.DataSource.DataSet;
     lista_campi[Length(lista_campi)-1].CampoDataSet:=fieldname;
     lista_campi[Length(lista_campi)-1].DataSetList:=DataSetList;
     lista_campi[Length(lista_campi)-1].ChiaveLista:=keylist;
     lista_campi[Length(lista_campi)-1].CampoLista:=fieldlist;

end;

procedure TDBGrid.UpdateColumnDBCombo();
var
   i : integer;
begin

  AddDBComboBoxToGridNew(lista_campi);

  for i:=0 to Length(lista_campi)-1 do
  begin
       SetLength(indici,Length(indici)+1);

        if GetIndexOfColumnsFromTitle(Self.DataSource.DataSet, lista_campi[i].CampoDataSet, lista_campi[i].NomeNuovoCampo, indici[i]) then
        begin
             //non faccio nulla, mi serve solo per salvarmi i valori in indici
        end;
  end;

  ExchangeColumns(Self.DataSource.DataSet,indici);

end;

procedure TDBGrid.ClearColumnsCombo;
var
   i: integer;
begin

     for i:=0 to Length(ListOfComboColumn)-1 do
     begin
          SetLength(ListOfComboColumn[i].indexs,0);
          SetLength(ListOfComboColumn[i].values,0);
     end;

     SetLength(ListOfComboColumn,0);

end;

function TDBGrid.GetFilterText(aCol: integer): string;
var
   idx: integer;
begin
     idx:=GetIdxFromTitleFilterTextValues(aCol);
     if idx=-1 then
        result:=''
     else
         result:=Self.TitleFilterTextValues[idx].filter;
end;

procedure TDBGrid.HideOtherComponent();
begin
     if Self.Edit1.Visible then
     begin
          //edit per filtraggio colonna
          Self.Edit1.Visible:=false;
     end;

     if Self.ComboBoxF.Visible then
     begin
          //CheckBox per filtraggio colonna
          Self.ComboBoxF.Visible:=false;
     end;

     if Self.ComboBox1.Visible then
     begin
       //combobox per cambiare il valore dei campi
       Self.OnComboBox1EditingDone(Self.ComboBox1);
       Self.ComboBox1.Visible:=false;
     end;

end;

function TDBGrid.ColumnIsCombo(aFieldName: string; var Style: TComboBoxStyle
  ): boolean;
var
   i: integer;
   ret: boolean;
begin
     i:=0;
     ret:=false;
     while (i<Length(ListOfComboColumn))and(ret=false) do
     begin
          if trim(uppercase(ListOfComboColumn[i].fieldName)) = trim(uppercase(aFieldName)) then
          begin
               Style:=ListOfComboColumn[i].Style;
               ret:=true;
          end
          else
              Inc(i);
     end;
     result:=ret;
end;

procedure TDBGrid.OnChangeComboFilter(Sender: TObject);
var
   colonna, idx,lCol: integer;
begin
     if dgIndicator in Self.Options then
        lCol:=1
     else
         lCol:=0;
     colonna:=ptrint( TEdit(Sender).Tag );
     idx:=Self.GetIdxFromTitleFilterTextValues(colonna);
     if idx <>-1 then
     begin
           Self.TitleFilterTextValues[idx].filter:=TEdit(Sender).Text;
           Self.TitleFilterTextValues[idx].label_filter:=TEdit(Sender).Text;
           Self.DrawRow(0);
           Self.InvalidateRow(0);
           Self.FOnTextTitleChange(Sender,colonna, Self.DataSource.DataSet.Fields[colonna-lCol].FieldName,TComboBox(Sender).Text);
     end;
end;

end.

